/* Name: Richard Eisenberg
 * File: MP3.java
 * Desc: Represents an MP3 that can be played[
 */
package edu.brynmawr;

import java.io.*;
import javazoom.jl.player.*;
import javazoom.jl.decoder.*;
import java.net.*;
import java.security.*;
import java.util.*;

public class MP3
{
	private class Runner extends Thread
	{
		private boolean loop; // continue to play mp3 even after it's done.
		private ByteArrayInputStream stream; // where does the data come from?
		private Player player; // the MP3 player behind the scenes

		public Runner(ByteArrayInputStream input, boolean myLoop)
		{
			stream = input;
			loop = myLoop;
		}

		public void setLoop(boolean loop)
		{
			this.loop = loop;
		}

		public void run()
		{
			try
			{
				do
				{
					if(loop)
					{
						// play in a loop
						while(loop)
						{
							stream.reset();
							player = new Player(stream);
							player.play();
							player.close();
						}
					}
					else
					{
						// otherwise, just play
						player = new Player(stream);
						player.play();
						player.close();
					}
				} while (loop);
			}
			catch(JavaLayerException e)
			{
				throw new IllegalArgumentException(e.toString());
			}
		}

		public void stopPlaying()
		{
			loop = false;
			player.close();
		}
	}
	
	private static URL codeBase;
	static
	{
		try
		{
			codeBase = new URL("file:///" + new File("").getAbsolutePath() + "/");
		}
		catch(MalformedURLException e)
		{
			System.out.println("Malformed URL during MP3 initialization.");
		}
		catch(AccessControlException e)
		{
			// that's OK.
		}
	}
	
	public static void setCodeBase(URL base)
	{
		codeBase = base;
	}

	private byte[] mp3data; // the raw MP3 data
	private URL url; // the file we are using
	private boolean loaded; // are the data loaded?
	private Runner runner; // the thread behind the scenes

	/* Attaches this object to a file on the system of the given filename. */
	public MP3(String filename)
	{
		try
		{
			init(new URL(codeBase, filename));
		}
		catch(MalformedURLException e)
		{
			System.out.println("Error forming a URL from " + codeBase + " and " + filename + ".");
		}
	}

	/* Attaches this object to a File on the system. */
	public MP3(URL myUrl)
	{
		init(myUrl);
	}
	
	private void init(URL myUrl)
	{
		url = myUrl;
		loaded = false;	
	}

	/* Loads the MP3 */
	public void load()
	{
		if(loaded)
		{
			return; // already done
		}

		// otherwise, read in the file
		ArrayList<Byte> data = new ArrayList<Byte>();
		
		try
		{
			InputStream inputStream = url.openStream();
			
			int oneByte;
			while((oneByte = inputStream.read()) >= 0)
			{
				data.add((byte)oneByte);
			}
			inputStream.close();
		}
		catch(IOException e)
		{
			throw new IllegalArgumentException(e.toString());
		}

		mp3data = new byte[data.size()];
		for(int i = 0; i < mp3data.length; i++)
		{
			mp3data[i] = data.get(i);
		}
		
		loaded = true;
	}

	/* Plays the MP3 once. */
	public void play() { play(false); }

	/* Loops the MP3. */
	public void loop() { play(true); }

	/* Plays the MP3 in a background thread, either once or many times. */
	public void play(boolean loop)
	{
		if(!loaded)
		{
			load();
		}

		if(runner != null)
		{
			// we're already looping
			runner.setLoop(loop);
		}
		else
		{
		// start it:
			runner = new Runner(new ByteArrayInputStream(mp3data), loop);
			runner.start();
		}
	}

	/* Stops the MP3 from running. */
	public void stop()
	{
		if(runner == null)
		{
			return;
		}

		runner.stopPlaying();
		runner = null;
	}
}
